@@ -3,15 +3,17 @@ import pycountry
 from . import DATA_DIR
 import itertools
 import csv
+import numpy as np
 import xarray
 
 pd.options.mode.chained_assignment = None
 
 REGION_MAPPING_FILEPATH = DATA_DIR / "regionmappingH12.csv"
 REMIND_ELEC_MARKETS = DATA_DIR / "remind_electricity_markets.csv"
+IEA_DIESEL_SHARE = DATA_DIR / "diesel_share_oecd.csv"
 
 
-def extract_biofuel_shares_from_REMIND(fp, remind_region, years):
+def extract_biofuel_shares_from_REMIND(fp, remind_region, years, allocate_all_synfuel=False):
     """
     This function extracts biofuel shares from a REMIND file provided.
 
@@ -20,6 +22,8 @@ def extract_biofuel_shares_from_REMIND(fp, remind_region, years):
     :param remind_region: REMIND region for which to extract the biofuel shares
     :type remind_region: str
     :param years: the list of years for which to extract biofuel shares
+    :param allocate_all_synfuel: Temporary workaround. If True, then all synfuel in the transport sector
+    is allocated to passenger cars.
     :return: a dictionary that contains fuel types as keys and lists of fuel shares as values
     """
 
@@ -30,10 +34,37 @@ def extract_biofuel_shares_from_REMIND(fp, remind_region, years):
     df = df.loc[df["Region"] == remind_region]
     df = df.loc[:, : str(2050)]
     df["Variable"] = df["Variable"].str.replace("|", "-")
-    var = ["FE-Transport-Liquids-Oil", "FE-Transport-Liquids-Biomass"]
 
-    df_liquids = df.loc[df["Variable"].isin(var), :]
-    df_liquids.iloc[:, 3:] /= df_liquids.iloc[:, 3:].sum(axis=0)
+    if allocate_all_synfuel:
+
+        # get shares of synthetic fuel
+        df_total = df.loc[df["Variable"] == "FE-Transport-Pass-Road-LDV-Liquids"]
+        df_total.index = df.loc[df["Variable"] == "SE-Liquids-Hydrogen"].index
+        share = np.clip(
+            (df.loc[df["Variable"] == "SE-Liquids-Hydrogen", "2005":].divide(
+                df_total.loc[:, "2005":], axis=0)
+            ).values,
+            0, 1)
+
+        var = ["FE-Transport-Liquids-Oil",
+               "FE-Transport-Liquids-Biomass"]
+
+        df_liquids = df.loc[df["Variable"].isin(var), :]
+        df_liquids.iloc[:, 3:] /= df_liquids.iloc[:, 3:].sum(axis=0)
+
+        df_liquids.loc[:, "2005":] *= (1 - share)
+        to_append = [remind_region, "FE-Transport-Liquids-Hydrogen", "EJ/yr"] + share[0].tolist()
+        a_series = pd.Series(to_append, index=df_liquids.columns)
+        df_liquids = df_liquids.append(a_series, ignore_index=True)
+
+    else:
+
+        var = ["FE-Transport-Liquids-Oil",
+               "FE-Transport-Liquids-Biomass",
+               "FE-Transport-Liquids-Hydrogen"]
+
+        df_liquids = df.loc[df["Variable"].isin(var), :]
+        df_liquids.iloc[:, 3:] /= df_liquids.iloc[:, 3:].sum(axis=0)
 
     var = ["FE-Transport-Gases-Non-Biomass", "FE-Transport-Gases-Biomass"]
 
@@ -43,24 +74,22 @@ def extract_biofuel_shares_from_REMIND(fp, remind_region, years):
     d_map_fuels = {
         "FE-Transport-Liquids-Oil": "liquid - fossil",
         "FE-Transport-Liquids-Biomass": "liquid - biomass",
+        "FE-Transport-Liquids-Hydrogen": "liquid - synfuel",
         "FE-Transport-Gases-Non-Biomass": "gas - fossil",
         "FE-Transport-Gases-Biomass": "gas - biomass",
     }
 
     new_df = pd.concat([df_liquids, df_gas])
     new_df["Variable"] = new_df["Variable"].map(d_map_fuels)
-
     new_df.columns = (
         new_df.columns[:3].tolist() + new_df.columns[3:].astype(int).tolist()
     )
 
     new_df = new_df.rename(columns={"Variable": "fuel_type"})
-
     new_df = new_df.groupby("fuel_type").sum()
 
     return new_df.to_xarray().to_array().interp(variable=years)
 
-
 def extract_electricity_mix_from_REMIND_file(fp, remind_region, years):
     """
     This function extracts electricity mixes from a REMIND file provided.
@@ -82,16 +111,13 @@ def extract_electricity_mix_from_REMIND_file(fp, remind_region, years):
     df = pd.read_csv(fp, delimiter=";", index_col=["Region", "Variable", "Unit"]).drop(
         columns=["Model", "Scenario"]
     )
+
     df = df.reset_index()
     df = df.loc[df["Region"] == remind_region]
     df = df.loc[:, : str(2050)]
-
     df = df.loc[df["Variable"].isin(electricity_markets.values())]
-
     df["Variable"] = df["Variable"].map(rev_tech)
-
     df.columns = df.columns[:3].tolist() + df.columns[3:].astype(int).tolist()
-
     df.iloc[:, 3:] /= df.iloc[:, 3:].sum(axis=0)
 
     d_var = {
@@ -146,8 +172,7 @@ def extract_electricity_mix_from_REMIND_file(fp, remind_region, years):
 
     return df.to_xarray().to_array().interp(variable=years).values
 
-
-def create_fleet_composition_from_REMIND_file(fp, remind_region, fleet_year=2020):
+def create_fleet_composition_from_REMIND_file(fp, remind_region, fleet_year=2020, normalized=True):
     """
     This function creates a consumable fleet composition array from a CSV file.
     The array returned is consumed by `InventoryCalculation`.
@@ -162,6 +187,17 @@ def create_fleet_composition_from_REMIND_file(fp, remind_region, fleet_year=2020
     :rtype: xarray.DataArray
     """
 
+    # Load diesel shares from IEA on OECD countries
+    share_diesel = pd.read_csv(IEA_DIESEL_SHARE,
+                               sep=";", usecols=range(1, 10), index_col=[0])
+    share_diesel.columns = [int(c) for c in share_diesel.columns]
+    shares = pd.concat([
+        share_diesel,
+        pd.DataFrame(columns=range(2019, 2051)),
+        pd.DataFrame(columns=range(2000, 2011))])
+    shares = shares.T.fillna(method="ffill")
+    shares = shares.fillna(method="bfill")
+
     # Read the fleet composition CSV file
     df = pd.read_csv(fp, delimiter=",")
     df = df.fillna(0)
@@ -183,6 +219,8 @@ def create_fleet_composition_from_REMIND_file(fp, remind_region, fleet_year=2020
     d_iso_2_remind = get_iso_2_to_REMIND_map()
     df["REMIND_region"] = df["iso_2"].map(d_iso_2_remind)
 
+
+
     # Filter out rows for which no REMIND region equivalence can be found.
     if len(df.loc[df["REMIND_region"].isnull(), "iso_2"].unique()) > 0:
         print(
@@ -210,9 +248,9 @@ def create_fleet_composition_from_REMIND_file(fp, remind_region, fleet_year=2020
     d_map_tech = {
         "BEV": "BEV",
         "FCEV": "FCEV",
-        "Hybrid Electric": "PHEV-p",
-        "Hybrid Liquids": "HEV-p",
-        "Liquids": "ICEV-p",
+        "Hybrid Electric": "PHEV",
+        "Hybrid Liquids": "HEV",
+        "Liquids": "ICEV",
         "NG": "ICEV-g",
     }
 
@@ -222,31 +260,76 @@ def create_fleet_composition_from_REMIND_file(fp, remind_region, fleet_year=2020
     # Clean up
     df["variable"] = df["variable"].str.replace(r"\D", "").astype(int)
 
+    # Rename `variable`, since it is a built-in name used by `xarray`
+    df = df.rename(columns={"variable": "vintage_year"})
+    df = df.loc[df["year"] == fleet_year]
+    df = df.loc[df["REMIND_region"] == remind_region]
+
+
+
+    # Associate diesel shares
+    def get_diesel_factor(row):
+
+        if row["iso_2"] in shares.columns:
+            return float(shares.loc[row["vintage_year"], row["iso_2"]])
+        else:
+            return 0
+
+    df["diesel_factor"] = df.apply(get_diesel_factor, axis=1)
+
+    new_df_p = df.loc[df["powertrain"].isin(("HEV", "PHEV", "ICEV"))]
+    new_df_p.loc[:, "powertrain"] += "-p"
+    new_df_p.loc[:, "vintage_demand_vkm"] *= (1 - new_df_p.loc[:, "diesel_factor"])
+    new_df_d = df.loc[df["powertrain"].isin(("HEV", "PHEV", "ICEV"))]
+    new_df_d.loc[:, "powertrain"] += "-d"
+    new_df_d.loc[:, "vintage_demand_vkm"] *= new_df_p.loc[:, "diesel_factor"]
+
+    df = pd.concat([df, new_df_p, new_df_d])
+    df = df.loc[~df["powertrain"].isin(("HEV", "PHEV", "ICEV"))]
+
+
+
     # Filter out unecessary columns
     df = df[
         [
-            "REMIND_region",
             "powertrain",
             "size",
             "year",
-            "variable",
+            "vintage_year",
             "vintage_demand_vkm",
         ]
     ]
 
-    # Rename `variable`, since it is a built-in name used by `xarray`
-    df = df.rename(columns={"variable": "vintage_year"})
-    df = df.loc[df["year"]==fleet_year]
-
     if len(df)==0:
         raise ValueError("This fleet year is not available.")
 
-    # Turn the dataframe into a pivot table and filter out unwanted REMIND regions
-    df = df.pivot_table(
-        index=["REMIND_region", "powertrain", "size", "vintage_year"], columns=["year"]
-    )["vintage_demand_vkm"]
-    df = df.loc[(remind_region)]
 
+    # Distribute the transport demand of 2010 to anterior years
+
+    distr_km = {
+        2010: .25,
+        2009: .15,
+        2008: .15,
+        2007: .15,
+        2006: .15,
+        2005: .15
+    }
+
+    new_df = pd.DataFrame()
+    for y in range(2010, 2004, -1):
+        temp_df = df.loc[df["vintage_year"] == 2010]
+        temp_df["vintage_year"] = y
+        temp_df["vintage_demand_vkm"] *= distr_km[y]
+        new_df = pd.concat([new_df, temp_df])
+
+    df = df.loc[df["vintage_year"] != 2010]
+
+    df = pd.concat([df, new_df])
+
+    # Turn the dataframe into a pivot table
+    df = df.pivot_table(
+        index=["powertrain", "size", "vintage_year"], columns=["year"], aggfunc=np.sum
+    )["vintage_demand_vkm"]
 
 
     # The table needs to be square: so we add powertrain-size-vintage_year combinations and set their value to 0
@@ -265,8 +348,9 @@ def create_fleet_composition_from_REMIND_file(fp, remind_region, fleet_year=2020
     for row in [i for i in list(itertools.product(*a)) if i not in df.index]:
         df.loc[row] = 0
 
-    # The vkm values are normalized to 1, year-wise
-    df /= df.sum(axis=0)
+    if normalized:
+        # The vkm values are normalized to 1, year-wise
+        df /= df.sum(axis=0)
 
     # xarray.DataArray is returned
     return df.to_xarray().fillna(0).to_array()
